@@ -29,6 +29,8 @@
 #ifndef KMAINWINDOW_P_H
 #define KMAINWINDOW_P_H
 
+#include <kconfiggroup.h>
+
 #define K_D(Class) Class##Private * const d = k_func()
 
 class QObject;
@@ -43,18 +45,27 @@ public:
     bool autoSaveWindowSize:1;
     bool care_about_geometry:1;
     bool shuttingDown:1;
-    QString autoSaveGroup;
+    KConfigGroup autoSaveGroup;
     QTimer* settingsTimer;
     QRect defaultWindowSize;
     KHelpMenu *helpMenu;
     KMainWindow *q;
     QObject* dockResizeListener;
     QString dbusName;
+    bool letDirtySettings;
 
     void _k_shuttingDown();
+    // This slot will be called when the style KCM changes settings that need
+    // to be set on the already running applications.
+    void _k_slotSettingsChanged(int category);
 
     void init(KMainWindow *_q);
     void polish(KMainWindow *q);
+    enum CallCompression {
+        NoCompressCalls = 0,
+        CompressCalls
+    };
+    void setSettingsDirty(CallCompression callCompression = NoCompressCalls);
 };
 
 #endif
