@@ -264,11 +264,16 @@ int KTabWidget::tabBarWidthForMaxChars( int maxLength )
 
     int lw = fm.width( newTitle );
     int iw = 0;
-    if ( !tabBar()->tabIcon( i ).isNull() )
+    if ( !tabBar()->tabIcon( i ).isNull() ) {
       iw = tabBar()->tabIcon( i ).pixmap( style()->pixelMetric( QStyle::PM_SmallIconSize ), QIcon::Normal ).width() + 4;
-      x += ( tabBar()->style()->sizeFromContents( QStyle::CT_TabBarTab, 0L,
-           QSize( qMax( lw + hframe + iw, QApplication::globalStrut().width() ), 0 ),
-           this ) ).width();
+    }
+    if ( isCloseButtonEnabled() ) {
+      // FIXME: how to get the size of the close button directly from the tabBar()?
+      iw += KIconLoader::SizeSmall * 3 / 2;
+    }
+    x += ( tabBar()->style()->sizeFromContents( QStyle::CT_TabBarTab, 0L,
+         QSize( qMax( lw + hframe + iw, QApplication::globalStrut().width() ), 0 ),
+         this ) ).width();
   }
 
   return x;
@@ -276,14 +281,22 @@ int KTabWidget::tabBarWidthForMaxChars( int maxLength )
 
 QString KTabWidget::tabText( int index ) const
 {
-  if ( d->m_automaticResizeTabs ) {
-    if ( index >= 0 && index < count() )
-      return d->m_tabNames[ index ];
+    if ( d->m_automaticResizeTabs ) {
+        if (index >= 0 && index < count()) {
+            if (index >= d->m_tabNames.count()) {
+                // Ooops, the tab exists, but tabInserted wasn't called yet.
+                // This can happen when inserting the first tab,
+                // and calling tabText from slotCurrentChanged,
+                // see KTabWidget_UnitTest.
+                const_cast<KTabWidget*>(this)->tabInserted(index);
+            }
+            return d->m_tabNames[ index ];
+        }
+        else
+            return QString();
+    }
     else
-      return QString();
-  }
-  else
-    return QTabWidget::tabText( index );
+        return QTabWidget::tabText( index );
 }
 
 void KTabWidget::setTabText( int index, const QString &text )
@@ -506,22 +519,36 @@ void KTabWidget::removeTab( int index )
 
 void KTabWidget::setHoverCloseButton( bool button )
 {
-  static_cast<KTabBar*>( tabBar() )->setHoverCloseButton( button );
+  // deprecated
+  setCloseButtonEnabled( button );
 }
 
 bool KTabWidget::hoverCloseButton() const
 {
-  return static_cast<KTabBar*>( tabBar() )->hoverCloseButton();
+  // deprecated
+  return false;
 }
 
 void KTabWidget::setHoverCloseButtonDelayed( bool delayed )
 {
-  static_cast<KTabBar*>( tabBar() )->setHoverCloseButtonDelayed( delayed );
+  // deprecated
+  Q_UNUSED( delayed );
 }
 
 bool KTabWidget::hoverCloseButtonDelayed() const
 {
-  return static_cast<KTabBar*>( tabBar() )->hoverCloseButtonDelayed();
+  // deprecated
+  return isCloseButtonEnabled();
+}
+
+void KTabWidget::setCloseButtonEnabled( bool enable )
+{
+  static_cast<KTabBar*>( tabBar() )->setCloseButtonEnabled( enable );
+}
+
+bool KTabWidget::isCloseButtonEnabled() const
+{
+  return static_cast<KTabBar*>( tabBar() )->isCloseButtonEnabled();
 }
 
 void KTabWidget::setAutomaticResizeTabs( bool enabled )
