@@ -44,7 +44,9 @@
 #include <QtGui/QTreeView>
 #include <QtGui/QPushButton>
 #include <QtGui/QProgressBar>
+#include <QtGui/QScrollBar>
 #include <QtGui/QSplitter>
+#include <QtGui/QWheelEvent>
 
 #include <kaction.h>
 #include <kapplication.h>
@@ -92,6 +94,7 @@ protected:
     virtual QStyleOptionViewItem viewOptions() const;
     virtual void dragEnterEvent(QDragEnterEvent* event);
     virtual void mousePressEvent(QMouseEvent *event);
+    virtual void wheelEvent(QWheelEvent *event);
 };
 
 KDirOperatorIconView::KDirOperatorIconView(QWidget *parent) :
@@ -105,6 +108,7 @@ KDirOperatorIconView::KDirOperatorIconView(QWidget *parent) :
     setDragDropMode(QListView::DragOnly);
     setVerticalScrollMode(QListView::ScrollPerPixel);
     setHorizontalScrollMode(QListView::ScrollPerPixel);
+    setEditTriggers(QAbstractItemView::NoEditTriggers);
 
     QFontMetrics metrics(viewOptions().font);
     const int fontHeight = metrics.height();
@@ -145,6 +149,22 @@ void KDirOperatorIconView::mousePressEvent(QMouseEvent *event)
     QListView::mousePressEvent(event);
 }
 
+void KDirOperatorIconView::wheelEvent(QWheelEvent *event)
+{
+    QListView::wheelEvent(event);
+
+    // apply the vertical wheel event to the horizontal scrollbar, as
+    // the items are aligned from left to right
+    if (event->orientation() == Qt::Vertical) {
+        QWheelEvent horizEvent(event->pos(),
+                               event->delta(),
+                               event->buttons(),
+                               event->modifiers(),
+                               Qt::Horizontal);
+        QApplication::sendEvent(horizontalScrollBar(), &horizEvent);
+    }
+}
+
 class KDirOperator::Private
 {
 public:
@@ -187,7 +207,6 @@ public:
     void _k_slotSelectionChanged();
     void _k_openContextMenu(const QPoint&);
     void _k_triggerPreview(const QModelIndex&);
-    void _k_cancelPreview();
     void _k_showPreview();
     void _k_slotSplitterMoved(int, int);
     void _k_assureVisibleSelection();
@@ -198,6 +217,8 @@ public:
     QStack<KUrl*> backStack;    ///< Contains all URLs you can reach with the back button.
     QStack<KUrl*> forwardStack; ///< Contains all URLs you can reach with the forward button.
 
+    QModelIndex lastHoveredIndex;
+
     KDirLister *dirLister;
     KUrl currUrl;
 
@@ -222,7 +243,6 @@ public:
     QProgressBar *progressBar;
 
     KPreviewWidgetBase *preview;
-    QTimer *previewTimer;
     KUrl previewUrl;
     int previewWidth;
 
@@ -248,7 +268,6 @@ KDirOperator::Private::Private(KDirOperator *_parent) :
     proxyModel(0),
     progressBar(0),
     preview(0),
-    previewTimer(0),
     previewUrl(),
     previewWidth(0),
     leftButtonPressed(false),
@@ -299,10 +318,6 @@ KDirOperator::KDirOperator(const KUrl& _url, QWidget *parent) :
             this, SLOT(_k_slotSplitterMoved(int, int)));
 
     d->preview = 0;
-    d->previewTimer = new QTimer(this);
-    d->previewTimer->setSingleShot(true);
-    connect(d->previewTimer, SIGNAL(timeout()),
-            this, SLOT(_k_showPreview()));
 
     d->mode = KFile::File;
     d->viewKind = KFile::Simple;
@@ -384,28 +399,37 @@ void KDirOperator::resetCursor()
 
 void KDirOperator::sortByName()
 {
+    d->sorting = QDir::Name;
     d->actionCollection->action("by name")->setChecked(true);
 }
 
 void KDirOperator::sortBySize()
 {
+    d->sorting = (d->sorting & ~QDir::SortByMask) | QDir::Size;
     d->actionCollection->action("by size")->setChecked(true);
 }
 
 void KDirOperator::sortByDate()
 {
+    d->sorting = (d->sorting & ~QDir::SortByMask) | QDir::Time;
     d->actionCollection->action("by date")->setChecked(true);
 }
 
 void KDirOperator::sortByType()
 {
+    d->sorting = (d->sorting & ~QDir::SortByMask) | QDir::Type;
     d->actionCollection->action("by type")->setChecked(true);
 }
 
 void KDirOperator::sortReversed()
 {
-    QAction* action = d->actionCollection->action("descending");
-    action->setChecked(!action->isChecked());
+    if (d->sorting & QDir::Reversed) {
+        d->sorting = d->sorting & ~QDir::Reversed;
+        d->actionCollection->action("descending")->setChecked(false);
+    } else {
+        d->sorting = d->sorting | QDir::Reversed;
+        d->actionCollection->action("descending")->setChecked(true);
+    }
 }
 
 void KDirOperator::toggleDirsFirst()
@@ -463,7 +487,7 @@ KFileItemList KDirOperator::selectedItems() const
     const QItemSelection selection = d->proxyModel->mapSelectionToSource(d->itemView->selectionModel()->selection());
 
     const QModelIndexList indexList = selection.indexes();
-    foreach(QModelIndex index, indexList) {
+    foreach(const QModelIndex &index, indexList) {
         KFileItem item = d->dirModel->itemForIndex(index);
         if (!item.isNull()) {
             itemList.append(item);
@@ -550,47 +574,36 @@ void KDirOperator::Private::_k_togglePreview(bool on)
     } else if (preview != 0) {
         viewKind = viewKind & ~KFile::PreviewContents;
         preview->hide();
-        previewTimer->stop();
     }
 }
 
 void KDirOperator::Private::_k_slotSortByName()
 {
-    sorting = QDir::Name;
-    actionCollection->action("by name")->setChecked(true);
+    parent->sortByName();
     triggerSorting();
 }
 
 void KDirOperator::Private::_k_slotSortBySize()
 {
-    sorting = (sorting & ~QDir::SortByMask) | QDir::Size;
-    actionCollection->action("by size")->setChecked(true);
+    parent->sortBySize();
     triggerSorting();
 }
 
 void KDirOperator::Private::_k_slotSortByDate()
 {
-    sorting = (sorting & ~QDir::SortByMask) | QDir::Time;
-    actionCollection->action("by date")->setChecked(true);
+    parent->sortByDate();
     triggerSorting();
 }
 
 void KDirOperator::Private::_k_slotSortByType()
 {
-    sorting = (sorting & ~QDir::SortByMask) | QDir::Type;
-    actionCollection->action("by type")->setChecked(true);
+    parent->sortByType();
     triggerSorting();
 }
 
 void KDirOperator::Private::_k_slotSortReversed()
 {
-    if (sorting & QDir::Reversed) {
-        sorting = sorting & ~QDir::Reversed;
-        actionCollection->action("descending")->setChecked(false);
-    } else {
-        sorting = sorting | QDir::Reversed;
-        actionCollection->action("descending")->setChecked(true);
-    }
+    parent->sortReversed();
     triggerSorting();
 }
 
@@ -638,20 +651,20 @@ void KDirOperator::mkdir()
                                            i18n("Create new folder in:\n%1" ,  where),
                                            name, &ok, this);
     if (ok)
-        mkdir(KIO::encodeFileName(folder), true);
+        KDirOperator::mkdir(KIO::encodeFileName(folder), true);
 }
 
 bool KDirOperator::mkdir(const QString& directory, bool enterDirectory)
 {
     // Creates "directory", relative to the current directory (d->currUrl).
-    // The given path may contain any number directories, existant or not.
+    // The given path may contain any number directories, existent or not.
     // They will all be created, if possible.
 
     bool writeOk = false;
     bool exists = false;
     KUrl url(d->currUrl);
 
-    QStringList dirs = directory.split(QDir::separator(), QString::SkipEmptyParts);
+    const QStringList dirs = directory.split(QDir::separator(), QString::SkipEmptyParts);
     QStringList::ConstIterator it = dirs.begin();
 
     for (; it != dirs.end(); ++it) {
@@ -660,7 +673,7 @@ bool KDirOperator::mkdir(const QString& directory, bool enterDirectory)
         writeOk = !exists && KIO::NetAccess::mkdir(url, topLevelWidget());
     }
 
-    if (exists) { // url was already existant
+    if (exists) { // url was already existent
         KMessageBox::sorry(d->itemView, i18n("A file or folder named %1 already exists.", url.pathOrUrl()));
         enterDirectory = false;
     } else if (!writeOk) {
@@ -690,7 +703,7 @@ KIO::DeleteJob * KDirOperator::del(const KFileItemList& items,
 
     KUrl::List urls;
     QStringList files;
-    foreach (const KFileItem item, items) {
+    foreach (const KFileItem &item, items) {
         const KUrl url = item.url();
         urls.append(url);
         files.append(url.pathOrUrl());
@@ -748,7 +761,7 @@ KIO::CopyJob * KDirOperator::trash(const KFileItemList& items,
 
     KUrl::List urls;
     QStringList files;
-    foreach (const KFileItem item, items) {
+    foreach (const KFileItem &item, items) {
         const KUrl url = item.url();
         urls.append(url);
         files.append(url.pathOrUrl());
@@ -889,12 +902,17 @@ void KDirOperator::setUrl(const KUrl& _newurl, bool clearforward)
     if (!Private::isReadable(newurl)) {
         // maybe newurl is a file? check its parent directory
         newurl.cd(QLatin1String(".."));
-        if (!Private::isReadable(newurl)) {
+        KIO::UDSEntry entry;
+        bool res = KIO::NetAccess::stat(newurl, entry, this);
+        KFileItem i(entry, newurl);
+        if ((!res || !Private::isReadable(newurl)) && i.isDir()) {
             resetCursor();
             KMessageBox::error(d->itemView,
                                i18n("The specified folder does not exist "
                                     "or was not readable."));
             return;
+        } else if (!i.isDir()) {
+            return;
         }
     }
 
@@ -980,7 +998,9 @@ void KDirOperator::Private::triggerSorting()
     QTreeView* treeView = qobject_cast<QTreeView*>(itemView);
     if (treeView != 0) {
         QHeaderView* headerView = treeView->header();
+        headerView->blockSignals(true);
         headerView->setSortIndicator(sortColumn(), sortOrder());
+        headerView->blockSignals(false);
     }
 
     _k_assureVisibleSelection();
@@ -1139,15 +1159,57 @@ void KDirOperator::changeEvent(QEvent *event)
     }
 }
 
+bool KDirOperator::eventFilter(QObject *watched, QEvent *event)
+{
+    Q_UNUSED(watched);
+
+    // If we are not hovering any items, check if there is a current index
+    // set. In that case, we show the preview of that item.
+    if ((event->type() == QEvent::MouseMove) && (d->preview != 0)) {
+        const QModelIndex hoveredIndex = d->itemView->indexAt(d->itemView->viewport()->mapFromGlobal(QCursor::pos()));
+
+        if (d->lastHoveredIndex == hoveredIndex)
+            return false;
+
+        d->lastHoveredIndex = hoveredIndex;
+
+        const QModelIndex focusedIndex = d->itemView->selectionModel() ? d->itemView->selectionModel()->currentIndex()
+                                                                       : QModelIndex();
+
+        if (!hoveredIndex.isValid() && focusedIndex.isValid() &&
+            d->itemView->selectionModel()->isSelected(focusedIndex) &&
+            (d->lastHoveredIndex != focusedIndex)) {
+            const QModelIndex sourceFocusedIndex = d->proxyModel->mapToSource(focusedIndex);
+            const KFileItem item = d->dirModel->itemForIndex(sourceFocusedIndex);
+            if (!item.isNull()) {
+                d->preview->showPreview(item.url());
+            }
+        }
+    }
+    else if ((event->type() == QEvent::MouseButtonRelease) && (d->preview != 0)) {
+        const QModelIndex hoveredIndex = d->itemView->indexAt(d->itemView->viewport()->mapFromGlobal(QCursor::pos()));
+        const QModelIndex focusedIndex = d->itemView->selectionModel() ? d->itemView->selectionModel()->currentIndex()
+                                                                       : QModelIndex();
+
+        if (((!focusedIndex.isValid()) ||
+             !d->itemView->selectionModel()->isSelected(focusedIndex)) &&
+            (!hoveredIndex.isValid())) {
+            d->preview->clearPreview();
+        }
+    }
+
+    return false;
+}
+
 bool KDirOperator::Private::checkPreviewInternal() const
 {
-    QStringList supported = KIO::PreviewJob::supportedMimeTypes();
+    const QStringList supported = KIO::PreviewJob::supportedMimeTypes();
     // no preview support for directories?
     if (parent->dirOnlyMode() && supported.indexOf("inode/directory") == -1)
         return false;
 
     QStringList mimeTypes = dirLister->mimeFilters();
-    QStringList nameFilter = dirLister->nameFilter().split(" ", QString::SkipEmptyParts);
+    const QStringList nameFilter = dirLister->nameFilter().split(" ", QString::SkipEmptyParts);
 
     if (mimeTypes.isEmpty() && nameFilter.isEmpty() && !supported.isEmpty())
         return true;
@@ -1156,7 +1218,7 @@ bool KDirOperator::Private::checkPreviewInternal() const
         r.setPatternSyntax(QRegExp::Wildcard);   // the "mimetype" can be "image/*"
 
         if (!mimeTypes.isEmpty()) {
-            QStringList::Iterator it = supported.begin();
+            QStringList::ConstIterator it = supported.begin();
 
             for (; it != supported.end(); ++it) {
                 r.setPattern(*it);
@@ -1183,7 +1245,7 @@ bool KDirOperator::Private::checkPreviewInternal() const
 
                 // the "mimetypes" we get from the PreviewJob can be "image/*"
                 // so we need to check in wildcard mode
-                QStringList::Iterator it2 = supported.begin();
+                QStringList::ConstIterator it2 = supported.begin();
                 for (; it2 != supported.end(); ++it2) {
                     r.setPattern(*it2);
                     if (r.indexIn(mime) != -1) {
@@ -1305,6 +1367,8 @@ void KDirOperator::setView(QAbstractItemView *view)
     d->itemView = view;
     d->itemView->setModel(d->proxyModel);
 
+    view->viewport()->installEventFilter(this);
+
     KFileItemDelegate *delegate = new KFileItemDelegate(d->itemView);
     d->itemView->setItemDelegate(delegate);
     d->itemView->viewport()->setAttribute(Qt::WA_Hover);
@@ -1324,8 +1388,6 @@ void KDirOperator::setView(QAbstractItemView *view)
             this, SLOT(_k_openContextMenu(const QPoint&)));
     connect(d->itemView, SIGNAL(entered(const QModelIndex&)),
             this, SLOT(_k_triggerPreview(const QModelIndex&)));
-    connect(d->itemView, SIGNAL(viewportEntered()),
-            this, SLOT(_k_cancelPreview()));
     // assure that the sorting state d->sorting matches with the current action
     const bool descending = d->actionCollection->action("descending")->isChecked();
     if (!descending && d->sorting & QDir::Reversed) {
@@ -1399,7 +1461,7 @@ void KDirOperator::setDirLister(KDirLister *lister)
 
 void KDirOperator::selectDir(const KFileItem &item)
 {
-    setUrl(item.url(), true);
+    setUrl(item.targetUrl(), true);
 }
 
 void KDirOperator::selectFile(const KFileItem &item)
@@ -1439,9 +1501,8 @@ void KDirOperator::setCurrentItem(const KFileItem& item)
         if (!item.isNull()) {
             const QModelIndex dirIndex = d->dirModel->indexForItem(item);
             const QModelIndex proxyIndex = d->proxyModel->mapFromSource(dirIndex);
-            selModel->setCurrentIndex(proxyIndex, QItemSelectionModel::Select);
-            selModel->select(proxyIndex, QItemSelectionModel::Select);
-            d->_k_assureVisibleSelection();
+            selModel->setCurrentIndex(proxyIndex, QItemSelectionModel::SelectCurrent);
+            QMetaObject::invokeMethod(this, "_k_assureVisibleSelection", Qt::QueuedConnection);
         }
     }
 }
@@ -1476,7 +1537,7 @@ void KDirOperator::prepareCompletionObjects()
 
     if (d->completeListDirty) {   // create the list of all possible completions
         const KFileItemList itemList = d->dirLister->items();
-        foreach (const KFileItem item, itemList) {
+        foreach (const KFileItem &item, itemList) {
             d->completion.addItem(item.name());
             if (item.isDir()) {
                 d->dirCompletion.addItem(item.name());
@@ -1599,11 +1660,9 @@ void KDirOperator::setupActions()
     // TODO: QAbstractItemView does not offer an action collection. Provide
     // an interface to add a custom action collection.
 
-#if (QT_VERSION >= QT_VERSION_CHECK(4, 4, 0))
-    d->actionCollection->addAssociatedWidget(topLevelWidget());
+    d->actionCollection->addAssociatedWidget(parentWidget());
     foreach (QAction* action, d->actionCollection->actions())
       action->setShortcutContext(Qt::WidgetWithChildrenShortcut);
-#endif
 }
 
 void KDirOperator::setupMenu()
@@ -1935,7 +1994,9 @@ void KDirOperator::Private::_k_slotPressed(const QModelIndex&)
     // Remember whether the left mouse button has been pressed, to prevent
     // that a right-click on an item opens an item (see _k_slotClicked(),
     // _k_slotDoubleClicked() and _k_openContextMenu()).
-    leftButtonPressed = (QApplication::mouseButtons() & Qt::LeftButton);
+    const Qt::KeyboardModifiers modifiers = QApplication::keyboardModifiers();
+    leftButtonPressed = (QApplication::mouseButtons() & Qt::LeftButton) &&
+                        !(modifiers & Qt::ShiftModifier) && !(modifiers & Qt::ControlModifier);
 }
 
 void KDirOperator::Private::_k_slotClicked(const QModelIndex& index)
@@ -1944,7 +2005,10 @@ void KDirOperator::Private::_k_slotClicked(const QModelIndex& index)
         return;
     }
 
-    _k_slotActivated(index);
+    if (!parent->onlyDoubleClickSelectsFiles())
+        _k_slotDoubleClicked(index);
+    else
+        _k_slotActivated(index);
 }
 
 void KDirOperator::Private::_k_slotActivated(const QModelIndex& index)
@@ -1953,7 +2017,8 @@ void KDirOperator::Private::_k_slotActivated(const QModelIndex& index)
     KFileItem item = dirModel->itemForIndex(dirIndex);
     bool selectDir = false;
 
-    if (item.isNull())
+    const Qt::KeyboardModifiers modifiers = QApplication::keyboardModifiers();
+    if (item.isNull() || (modifiers & Qt::ShiftModifier) || (modifiers & Qt::ControlModifier))
         return;
 
     if (item.isDir()) {
@@ -1966,7 +2031,7 @@ void KDirOperator::Private::_k_slotActivated(const QModelIndex& index)
     if (selectDir) {
         parent->selectDir(item);
     } else {
-        parent->highlightFile(item);
+        parent->selectFile(item);
     }
 }
 
@@ -2004,6 +2069,10 @@ void KDirOperator::Private::_k_slotSelectionChanged()
         KFileItem nullItem;
         parent->highlightFile(nullItem);
     }
+    else {
+        KFileItem selectedItem = parent->selectedItems().first();
+        parent->highlightFile(selectedItem);
+    }
 }
 
 void KDirOperator::Private::_k_openContextMenu(const QPoint& pos)
@@ -2031,16 +2100,13 @@ void KDirOperator::Private::_k_triggerPreview(const QModelIndex& index)
 
         if (!item.isDir()) {
             previewUrl = item.url();
-            previewTimer->start(300);
+            _k_showPreview();
+        } else {
+            preview->clearPreview();
         }
     }
 }
 
-void KDirOperator::Private::_k_cancelPreview()
-{
-    previewTimer->stop();
-}
-
 void KDirOperator::Private::_k_showPreview()
 {
     if (preview != 0) {
@@ -2065,9 +2131,8 @@ void KDirOperator::Private::_k_assureVisibleSelection()
 
     QItemSelectionModel* selModel = itemView->selectionModel();
     if (selModel->hasSelection()) {
-        const QModelIndexList list = selModel->selection().indexes();
-        const QModelIndex index = list.first();
-        itemView->scrollTo(index);
+        const QModelIndex index = selModel->currentIndex();
+        itemView->scrollTo(index, QAbstractItemView::PositionAtCenter);
         _k_triggerPreview(index);
     }
 }
