@@ -168,6 +168,7 @@ bool DOMNode::toBoolean(ExecState *) const
   onmove	DOMNode::OnMove			DontDelete
   onreset	DOMNode::OnReset		DontDelete
   onresize	DOMNode::OnResize		DontDelete
+  onscroll      DOMNode::OnScroll               DontDelete
   onselect	DOMNode::OnSelect		DontDelete
   onsubmit	DOMNode::OnSubmit		DontDelete
   onunload	DOMNode::OnUnload		DontDelete
@@ -321,6 +322,8 @@ JSValue* DOMNode::getValueProperty(ExecState *exec, int token) const
     return getListener(DOM::EventImpl::RESET_EVENT);
   case OnResize:
     return getListener(DOM::EventImpl::RESIZE_EVENT);
+  case OnScroll:
+    return getListener(DOM::EventImpl::SCROLL_EVENT);
   case OnSelect:
     return getListener(DOM::EventImpl::SELECT_EVENT);
   case OnSubmit:
@@ -335,11 +338,11 @@ JSValue* DOMNode::getValueProperty(ExecState *exec, int token) const
     if (doc->isHTMLDocument()) {
       HTMLCollectionImpl all(doc, HTMLCollectionImpl::DOC_ALL);
       unsigned long i = 0;
-      DOM::NodeImpl* n = all.firstItem();
-      for ( ; !n && n != impl(); n = all.nextItem() )
+      for (DOM::NodeImpl* n = all.firstItem(); n; n = all.nextItem() ) {
+        if (n == impl())
+            return jsNumber(i);
         ++i;
-      Q_ASSERT( n ); // node not in document.all !?
-      return jsNumber(i);
+      }
     }
     return jsUndefined();
   }
@@ -385,16 +388,16 @@ JSValue* DOMNode::getValueProperty(ExecState *exec, int token) const
       return rend ? jsNumber(rend->scrollHeight()) : jsUndefined();
     case ScrollLeft:
       if (rend && rend->layer()) {
-          if (rend->isRoot() && !rend->style()->hidesOverflow())
+          if (rend->isRoot() && !rend->hasOverflowClip())
               return jsNumber( node.document()->view() ? node.document()->view()->contentsX() : 0);
-          return jsNumber( rend->layer()->scrollXOffset() );
+          return jsNumber( rend->hasOverflowClip() ? rend->layer()->scrollXOffset() : 0 );
       }
       return jsNumber( 0 );
     case ScrollTop:
       if (rend && rend->layer()) {
-          if (rend->isRoot() && !rend->style()->hidesOverflow())
+          if (rend->isRoot() && !rend->hasOverflowClip())
               return jsNumber( node.document()->view() ? node.document()->view()->contentsY() : 0);
-          return jsNumber( rend->layer()->scrollYOffset() );
+          return jsNumber( rend->hasOverflowClip() ? rend->layer()->scrollYOffset() : 0 );
       }
       return jsNumber( 0 );
     default:
@@ -490,6 +493,9 @@ void DOMNode::putValueProperty(ExecState *exec, int token, JSValue* value, int /
   case OnResize:
     setListener(exec,DOM::EventImpl::RESIZE_EVENT,value);
     break;
+  case OnScroll:
+    setListener(exec,DOM::EventImpl::SCROLL_EVENT,value);
+    break;
   case OnSelect:
     setListener(exec,DOM::EventImpl::SELECT_EVENT,value);
     break;
@@ -513,7 +519,7 @@ void DOMNode::putValueProperty(ExecState *exec, int token, JSValue* value, int /
     switch (token) {
       case ScrollLeft:
         if (rend && rend->layer()) {
-          if (rend->style()->hidesOverflow())
+          if (rend->hasOverflowClip())
             rend->layer()->scrollToXOffset(value->toInt32(exec));
           else if (rend->isRoot()) {
             KHTMLView* sview = node.document()->view();
@@ -524,7 +530,7 @@ void DOMNode::putValueProperty(ExecState *exec, int token, JSValue* value, int /
         break;
       case ScrollTop:
         if (rend && rend->layer()) {
-          if (rend->style()->hidesOverflow())
+          if (rend->hasOverflowClip())
             rend->layer()->scrollToYOffset(value->toInt32(exec));
           else if (rend->isRoot()) {
             KHTMLView* sview = node.document()->view();
