@@ -62,7 +62,7 @@ namespace KJS {
            OnAbort, OnBlur, OnChange, OnClick, OnDblClick, OnDragDrop, OnError,
            OnFocus, OnKeyDown, OnKeyPress, OnKeyUp, OnLoad, OnMouseDown,
            OnMouseMove, OnMouseOut, OnMouseOver, OnMouseUp, OnMove, OnReset,
-           OnResize, OnSelect, OnSubmit, OnUnload,
+           OnResize, OnScroll, OnSelect, OnSubmit, OnUnload,
            OffsetLeft, OffsetTop, OffsetWidth, OffsetHeight, OffsetParent,
            ClientLeft, ClientTop, ClientWidth, ClientHeight, ScrollLeft, ScrollTop,
 	   ScrollWidth, ScrollHeight, SourceIndex, TextContent };
@@ -85,8 +85,9 @@ namespace KJS {
     JSValue* indexGetter(ExecState *exec, unsigned index);
     
     virtual bool getOwnPropertySlot(ExecState *exec, const Identifier& propertyName, PropertySlot& slot);
-    virtual JSValue* callAsFunction(ExecState *exec, JSObject* thisObj, const List&args);
+    virtual JSValue* callAsFunction(ExecState *exec, JSObject* thisObj, const List& args);
     virtual bool implementsCall() const { return true; }
+    virtual bool isFunctionType() const { return false; }
     virtual void getPropertyNames(ExecState* exec, PropertyNameArray& names);
 
     // no put - all read-only
