@@ -188,6 +188,8 @@ RenderWidget::RenderWidget(DOM::NodeImpl* node)
 {
     m_widget = 0;
     m_underMouse = 0;
+    m_buffer[0] = 0;
+    m_buffer[1] = 0;
     // a widget doesn't support being anonymous
     assert(!isAnonymous());
     m_view  = node->document()->view();
@@ -238,6 +240,8 @@ RenderWidget::~RenderWidget()
         if (m_ownsWidget)
             m_widget->deleteLater();
     }
+    delete m_buffer[0];
+    delete m_buffer[1];
 }
 
 class QWidgetResizeEvent : public QEvent
@@ -410,11 +414,15 @@ void RenderWidget::updateFromElement()
         if (!backgroundColor.isValid() && !style()->htmlHacks())
             backgroundColor = QColor(0,0,0,0);
 
-        // check if we have to paint our background and let it show through
+        // check if we have to paint our background and let it show through the widget
         bool trans = ( isRedirectedWidget() && style()->backgroundLayers() && 
                        style()->backgroundLayers()->hasImage() && !qobject_cast<KUrlRequester*>(m_widget) );
 
         QPalette pal(QApplication::palette(m_widget));
+        // We need a non-transparent version for widgets with popups (e.g. kcombobox). The popups must not let
+        // the background show through.
+        QPalette non_trans_pal = pal;
+
         if (color.isValid() || backgroundColor.isValid() || trans) {
             int contrast_ = KGlobalSettings::contrast();
             int highlightVal = 100 + (2*contrast_+4)*16/10;
@@ -428,6 +436,7 @@ void RenderWidget::updateFromElement()
                                        (backgroundColor == colorForCSSValue(CSS_VAL_BUTTONFACE)) );
             if (shouldChangeBgPal || trans) {
                 pal.setColor(widget()->backgroundRole(), trans ? QColor(0,0,0,0) : backgroundColor);
+                non_trans_pal.setColor(widget()->backgroundRole(), backgroundColor);
                 for ( int i = 0; i < QPalette::NColorGroups; ++i ) {
                     if (shouldChangeBgPal) {
                         pal.setColor( (QPalette::ColorGroup)i, QPalette::Window, backgroundColor );
@@ -435,9 +444,16 @@ void RenderWidget::updateFromElement()
                         pal.setColor( (QPalette::ColorGroup)i, QPalette::Dark, backgroundColor.dark(lowlightVal) );
                         pal.setColor( (QPalette::ColorGroup)i, QPalette::Mid, backgroundColor.dark(120) );
                         pal.setColor( (QPalette::ColorGroup)i, QPalette::Midlight, backgroundColor.light(110) );
+                        non_trans_pal.setColor( (QPalette::ColorGroup)i, QPalette::Window, backgroundColor );
+                        non_trans_pal.setColor( (QPalette::ColorGroup)i, QPalette::Light, backgroundColor.light(highlightVal) );
+                        non_trans_pal.setColor( (QPalette::ColorGroup)i, QPalette::Dark, backgroundColor.dark(lowlightVal) );
+                        non_trans_pal.setColor( (QPalette::ColorGroup)i, QPalette::Mid, backgroundColor.dark(120) );
+                        non_trans_pal.setColor( (QPalette::ColorGroup)i, QPalette::Midlight, backgroundColor.light(110) );
                     }
                     pal.setColor( (QPalette::ColorGroup)i, QPalette::Button, trans ? QColor(0,0,0,0):backgroundColor );
                     pal.setColor( (QPalette::ColorGroup)i, QPalette::Base, trans ? QColor(0,0,0,0):backgroundColor );
+                    non_trans_pal.setColor( (QPalette::ColorGroup)i, QPalette::Button, backgroundColor );
+                    non_trans_pal.setColor( (QPalette::ColorGroup)i, QPalette::Base, backgroundColor );
                 }
             }
             if ( color.isValid() ) {
@@ -452,31 +468,51 @@ void RenderWidget::updateFromElement()
                     { QPalette::Inactive, QPalette::Foreground },
                     { QPalette::Inactive, QPalette::ButtonText },
                     { QPalette::Inactive, QPalette::Text },
-                    { QPalette::Disabled,QPalette::ButtonText },
                     { QPalette::NColorGroups, QPalette::NColorRoles },
                 };
                 const ColorSet *set = toSet;
                 while( set->cg != QPalette::NColorGroups ) {
                     pal.setColor( set->cg, set->cr, color );
+                    non_trans_pal.setColor( set->cg, set->cr, color );
                     ++set;
                 }
 
-                QColor disfg = color;
-                int h, s, v;
-                disfg.getHsv( &h, &s, &v );
-                if (v > 128)
-                    // dark bg, light fg - need a darker disabled fg
-                    disfg = disfg.dark(lowlightVal);
-                else if (disfg != Qt::black)
-                    // light bg, dark fg - need a lighter disabled fg - but only if !black
-                    disfg = disfg.light(highlightVal);
-                else
-                    // black fg - use darkgray disabled fg
-                    disfg = Qt::darkGray;
-                pal.setColor(QPalette::Disabled,QPalette::Foreground,disfg);
+		QColor disfg = color;
+		int h, s, v;
+		disfg.getHsv( &h, &s, &v );
+		if (v > 128)
+		    // dark bg, light fg - need a darker disabled fg
+		    disfg = disfg.dark(lowlightVal);
+		else if (v > 64)
+		    // light bg, dark fg - need a lighter disabled fg - but only if !black
+		    disfg = disfg.light(highlightVal);
+		else
+		    // for really dark fg - use darkgray disabled fg,
+		    // as ::light is pretty useless in this range
+		    disfg = Qt::darkGray;
+		pal.setColor(QPalette::Disabled,QPalette::Foreground,disfg);
+		pal.setColor(QPalette::Disabled,QPalette::Text,disfg);
+		pal.setColor(QPalette::Disabled,QPalette::ButtonText,disfg);
+                non_trans_pal.setColor(QPalette::Disabled,QPalette::Foreground,disfg);
+                non_trans_pal.setColor(QPalette::Disabled,QPalette::Text,disfg);
+                non_trans_pal.setColor(QPalette::Disabled,QPalette::ButtonText,disfg);
             }
         }
-        m_widget->setPalette(pal);    
+        if (qobject_cast<QComboBox*>(m_widget)) {
+            m_widget->setPalette(pal);
+            // mmh great, there's no accessor for the popup... 
+            QList<QWidget*>l = qFindChildren<QWidget *>(m_widget, QString());
+            foreach(QWidget* w, l) {
+                if (qobject_cast<QAbstractScrollArea*>(w)) {
+                    // we have the listview, climb up to reach its container.
+                    assert( w->parentWidget() != m_widget );
+                    if (w->parentWidget())
+                        w->parentWidget()->setPalette(non_trans_pal);
+                }
+            }
+        } else {
+            m_widget->setPalette(pal);
+        }
 
         // Border:
         if (shouldPaintBorder())
@@ -605,8 +641,24 @@ void RenderWidget::paint(PaintInfo& paintInfo, int _tx, int _ty)
     else
         m_view->addChild( m_widget, xPos, -500000 +yPos);
     m_widget->show();
-    if (khtmlw)
-        paintWidget(paintInfo, m_widget, xPos, yPos);
+    if (khtmlw) {
+        if ( KHTMLView* v = qobject_cast<KHTMLView*>(m_widget) ) {
+            // our buffers are dedicated to scrollbars.
+            if (v->verticalScrollBar()->isVisible() && (!m_buffer[0] || v->verticalScrollBar()->size() != m_buffer[0]->size())) {
+                delete m_buffer[0];
+                m_buffer[0] = new QPixmap( v->verticalScrollBar()->size() );
+            }
+            if (v->horizontalScrollBar()->isVisible() && (!m_buffer[1] || v->horizontalScrollBar()->size() != m_buffer[1]->size())) {
+                delete m_buffer[1];
+                m_buffer[1] = new QPixmap( v->horizontalScrollBar()->size() );
+            }
+        } else if (!m_buffer[0] || (m_widget->size() != m_buffer[0]->size())) {
+            assert(!m_buffer[1]);
+            delete m_buffer[0];
+            m_buffer[0] = new QPixmap( m_widget->size() );
+        }
+        paintWidget(paintInfo, m_widget, xPos, yPos, m_buffer);
+    }
 }
 
 static void setInPaintEventFlag(QWidget* w, bool b = true, bool recurse=true)
@@ -631,7 +683,7 @@ static void setInPaintEventFlag(QWidget* w, bool b = true, bool recurse=true)
       }
 }
 
-static void copyWidget(const QRect& r, QPainter *p, QWidget *widget, int tx, int ty, bool buffered = false)
+static void copyWidget(const QRect& r, QPainter *p, QWidget *widget, int tx, int ty, bool buffered = false, QPixmap* buffer = 0)
 {
     if (r.isNull() || r.isEmpty() )
         return;
@@ -656,11 +708,14 @@ static void copyWidget(const QRect& r, QPainter *p, QWidget *widget, int tx, int
     if (buffered) {
         if (!widget->size().isValid())
             return;
-        pm = PaintBuffer::grab(widget->size());
-        // Qt 4.4 regression #1:
-        // QPainter::CompositionMode_Source is severly broken (cf. kde #160518)
+        // TT says Qt 4's widget painting hits an NVidia RenderAccel bug/shortcoming
+        // resulting in pixmap buffers being unsuitable for reuse by more than one widget.
         //
-        if (1 || !pm->hasAlphaChannel()) {
+        // Until a turnaround exist in Qt, we can't reliably use shared buffers.
+        // ###  pm = PaintBuffer::grab(widget->size());
+        assert( buffer );
+        pm = buffer;
+        if (!pm->hasAlphaChannel()) {
             pm->fill(Qt::transparent);
         } else {
             QPainter pp(pm);
@@ -669,7 +724,7 @@ static void copyWidget(const QRect& r, QPainter *p, QWidget *widget, int tx, int
         }
         d = pm;
     }
-    // Qt 4.4 regression #2: 
+    // Qt 4.4 regression #1: 
     // can't let a painter active on the view as Qt thinks it is opened on the *pixmap*
     // and prints "paint device can only be painted by one painter at a time" warnings.
     //
@@ -701,16 +756,16 @@ static void copyWidget(const QRect& r, QPainter *p, QWidget *widget, int tx, int
         // transfer results
         QPoint off(r.x(), r.y());
         p->drawPixmap(thePoint+off, static_cast<QPixmap&>(*d), r);
-        PaintBuffer::release(pm);
+        // ### PaintBuffer::release(pm);
     }
 }
 
-void RenderWidget::paintWidget(PaintInfo& pI, QWidget *widget, int tx, int ty)
+void RenderWidget::paintWidget(PaintInfo& pI, QWidget *widget, int tx, int ty, QPixmap* buffer[])
 {
     QPainter* const p = pI.p;
     allowWidgetPaintEvents = true;
 
-    // Qt 4.4 regression #3: 
+    // Qt 4.4 regression #2: 
     //    can't use QWidget::render to directly paint widgets on the view anymore.
     //    Results are unreliable for subrects, leaving blank squares. (cf. kde #158607)
     //
@@ -728,7 +783,7 @@ void RenderWidget::paintWidget(PaintInfo& pI, QWidget *widget, int tx, int ty)
             vbr &= r;
             vbr.translate( -of );
             if (vbr.isValid() && !vbr.isEmpty())
-                copyWidget(vbr, p, v->verticalScrollBar(), tx+of.x(), ty+of.y(), buffered);
+                copyWidget(vbr, p, v->verticalScrollBar(), tx+of.x(), ty+of.y(), buffered, buffer[0]);
         }
         if (v->horizontalScrollBar()->isVisible()) {
             QRect hbr = v->horizontalScrollBar()->rect();
@@ -737,13 +792,13 @@ void RenderWidget::paintWidget(PaintInfo& pI, QWidget *widget, int tx, int ty)
             hbr &= r;
             hbr.translate( -of );
             if (hbr.isValid() && !hbr.isEmpty())
-                copyWidget(hbr, p, v->horizontalScrollBar(), tx+ of.x(), ty+ of.y(), buffered);
+                copyWidget(hbr, p, v->horizontalScrollBar(), tx+ of.x(), ty+ of.y(), buffered, buffer[1]);
         }
         QRect vr = (r & v->viewport()->rect());
         if (vr.isValid() && !vr.isEmpty())
             v->render(p, vr, thePoint);
     } else {
-        copyWidget(r, p, widget, tx, ty, buffered);
+        copyWidget(r, p, widget, tx, ty, buffered, buffer[0]);
     }
     allowWidgetPaintEvents = false;
 }
@@ -988,6 +1043,10 @@ bool RenderWidget::handleEvent(const DOM::EventImpl& ev)
                 }
                 m_underMouse = target;
             }
+        } else if (target && ev.id() == EventImpl::MOUSEMOVE_EVENT) {
+            QMouseEvent evt(QEvent::MouseMove, p, Qt::NoButton,
+                            QApplication::mouseButtons(), QApplication::keyboardModifiers());
+            QApplication::sendEvent(target, &evt);
         }
 
         if (ev.id() == EventImpl::MOUSEDOWN_EVENT) {
