@@ -1,6 +1,6 @@
 /*
  * This file is part of the Nepomuk KDE project.
- * Copyright (C) 2006-2007 Sebastian Trueg <trueg@kde.org>
+ * Copyright (C) 2006-2008 Sebastian Trueg <trueg@kde.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -21,41 +21,34 @@
 #include "resourcemanager.h"
 #include "resourcedata.h"
 #include "tools.h"
-
+#include "nepomukmainmodel.h"
 #include "resource.h"
 
 #include <kglobal.h>
 #include <kdebug.h>
 #include <krandom.h>
 
-#include <Soprano/Client/DBusClient>
-#include <Soprano/Client/DBusModel>
 #include <Soprano/Node>
 #include <Soprano/Statement>
 #include <Soprano/Vocabulary/RDF>
 #include <Soprano/StatementIterator>
-#include <Soprano/Util/DummyModel>
+#include <Soprano/QueryResultIterator>
 
 using namespace Soprano;
 
 
-static const char* NEPOMUK_NAMESPACE = "http://nepomuk.kde.org/resources#";
-
 
 class Nepomuk::ResourceManager::Private
 {
 public:
     Private( ResourceManager* manager )
-        : client( "org.kde.NepomukServer" ),
-          mainModel( 0 ),
-          dummyModel( 0 ),
+        : mainModel( 0 ),
+          overrideModel( 0 ),
           m_parent(manager) {
     }
 
-    Soprano::Client::DBusClient client;
-    Soprano::Model* mainModel;
-
-    Soprano::Util::DummyModel* dummyModel;
+    Nepomuk::MainModel* mainModel;
+    Soprano::Model* overrideModel;
 
 private:
     ResourceManager* m_parent;
@@ -66,16 +59,11 @@ Nepomuk::ResourceManager::ResourceManager()
     : QObject(),
       d( new Private( this ) )
 {
-    connect( mainModel(), SIGNAL(statementsAdded()),
-             this, SLOT(slotStoreChanged()) );
-    connect( mainModel(), SIGNAL(statementsRemoved()),
-             this, SLOT(slotStoreChanged()) );
 }
 
 
 Nepomuk::ResourceManager::~ResourceManager()
 {
-    delete d->dummyModel;
     delete d;
 }
 
@@ -97,19 +85,31 @@ Nepomuk::ResourceManager* Nepomuk::ResourceManager::instance()
 
 int Nepomuk::ResourceManager::init()
 {
-    return ( initialized() ? 0 : -1 );
+    delete d->mainModel;
+    d->mainModel = new MainModel( this );
+    connect( d->mainModel, SIGNAL(statementsAdded()),
+             this, SLOT(slotStoreChanged()) );
+    connect( d->mainModel, SIGNAL(statementsRemoved()),
+             this, SLOT(slotStoreChanged()) );
+    return d->mainModel->isValid() ? 0 : -1;
 }
 
 
 bool Nepomuk::ResourceManager::initialized() const
 {
-    return d->client.isValid();
+    return d->mainModel && d->mainModel->isValid();
 }
 
 
 Nepomuk::Resource Nepomuk::ResourceManager::createResourceFromUri( const QString& uri )
 {
-    return Resource( uri, QString() );
+    return Resource( uri, QUrl() );
+}
+
+void Nepomuk::ResourceManager::removeResource( const QString& uri )
+{
+    Resource res( uri );
+    res.remove();
 }
 
 void Nepomuk::ResourceManager::notifyError( const QString& uri, int errorCode )
@@ -120,6 +120,12 @@ void Nepomuk::ResourceManager::notifyError( const QString& uri, int errorCode )
 
 
 QList<Nepomuk::Resource> Nepomuk::ResourceManager::allResourcesOfType( const QString& type )
+{
+    return allResourcesOfType( QUrl(type) );
+}
+
+
+QList<Nepomuk::Resource> Nepomuk::ResourceManager::allResourcesOfType( const QUrl& type )
 {
     QList<Resource> l;
 
@@ -134,11 +140,11 @@ QList<Nepomuk::Resource> Nepomuk::ResourceManager::allResourcesOfType( const QSt
         kDebug(300004) << " added local resources: " << l.count();
 
         Soprano::Model* model = mainModel();
-        Soprano::StatementIterator it = model->listStatements( Soprano::Statement( Soprano::Node(), Soprano::Vocabulary::RDF::type(), QUrl(type) ) );
+        Soprano::StatementIterator it = model->listStatements( Soprano::Statement( Soprano::Node(), Soprano::Vocabulary::RDF::type(), type ) );
 
         while( it.next() ) {
             Statement s = *it;
-            Resource res( s.subject().toString() );
+            Resource res( s.subject().uri() );
             if( !l.contains( res ) )
                 l.append( res );
         }
@@ -151,6 +157,12 @@ QList<Nepomuk::Resource> Nepomuk::ResourceManager::allResourcesOfType( const QSt
 
 
 QList<Nepomuk::Resource> Nepomuk::ResourceManager::allResourcesWithProperty( const QString& uri, const Variant& v )
+{
+    return allResourcesWithProperty( QUrl(uri), v );
+}
+
+
+QList<Nepomuk::Resource> Nepomuk::ResourceManager::allResourcesWithProperty( const QUrl& uri, const Variant& v )
 {
     QList<Resource> l;
 
@@ -168,18 +180,18 @@ QList<Nepomuk::Resource> Nepomuk::ResourceManager::allResourcesWithProperty( con
         // check remote data
         Soprano::Node n;
         if( v.isResource() ) {
-            n = QUrl( v.toResource().uri() );
+            n = v.toResource().resourceUri();
         }
         else {
             n = valueToRDFNode(v);
         }
 
         Soprano::Model* model = mainModel();
-        Soprano::StatementIterator it = model->listStatements( Soprano::Statement( Soprano::Node(), QUrl(uri), n ) );
+        Soprano::StatementIterator it = model->listStatements( Soprano::Statement( Soprano::Node(), uri, n ) );
 
         while( it.next() ) {
             Statement s = *it;
-            Resource res( s.subject().toString() );
+            Resource res( s.subject().uri() );
             if( !l.contains( res ) )
                 l.append( res );
         }
@@ -192,46 +204,62 @@ QList<Nepomuk::Resource> Nepomuk::ResourceManager::allResourcesWithProperty( con
 QString Nepomuk::ResourceManager::generateUniqueUri()
 {
     Soprano::Model* model = mainModel();
-    QUrl s;
+    QString s;
     while( 1 ) {
         // Should we use the Nepomuk localhost whatever namespace here?
-        s = NEPOMUK_NAMESPACE + KRandom::randomString( 20 );
-        if( !model->containsContext( s ) &&
-            !model->containsAnyStatement( Soprano::Statement( s, Soprano::Node(), Soprano::Node() ) ) &&
-            !model->containsAnyStatement( Soprano::Statement( Soprano::Node(), s, Soprano::Node() ) ) &&
-            !model->containsAnyStatement( Soprano::Statement( Soprano::Node(), Soprano::Node(), s ) ) )
-            return s.toString();
+        s = "nepomuk:/" + KRandom::randomString( 20 );
+        if ( !model->executeQuery( QString("ask where { { <%1> ?p1 ?o1 . } UNION { ?r2 <%1> ?o2 . } UNION { ?r3 ?p3 <%1> . } }")
+                                   .arg( s ), Soprano::Query::QueryLanguageSparql ).boolValue() ) {
+            return s;
+        }
     }
 }
 
 
 Soprano::Model* Nepomuk::ResourceManager::mainModel()
 {
+    if ( d->overrideModel ) {
+        return d->overrideModel;
+    }
+
     // make sure we are initialized
     if ( !initialized() ) {
-        delete d->mainModel;
-        d->mainModel = 0;
         init();
     }
 
-    if ( !d->mainModel ) {
-        d->mainModel = d->client.createModel( "main" );
-    }
-
-    if ( !d->mainModel ) {
-        if ( !d->dummyModel ) {
-            d->dummyModel = new Soprano::Util::DummyModel();
-        }
-        return d->dummyModel;
-    }
-
     return d->mainModel;
 }
 
 
 void Nepomuk::ResourceManager::slotStoreChanged()
 {
-    kDebug();
+//    kDebug();
+    Q_FOREACH( ResourceData* data, ResourceData::allResourceData()) {
+        data->invalidateCache();
+    }
+}
+
+
+void Nepomuk::ResourceManager::setOverrideMainModel( Soprano::Model* model )
+{
+    if ( d->overrideModel ) {
+        d->overrideModel->disconnect( this );
+    }
+
+    d->overrideModel = model;
+
+    if ( model ) {
+        connect( model, SIGNAL(statementsAdded()),
+                 this, SLOT(slotStoreChanged()) );
+        connect( model, SIGNAL(statementsRemoved()),
+                 this, SLOT(slotStoreChanged()) );
+    }
+
+    if ( d->mainModel ) {
+        d->mainModel->blockSignals( model != 0 );
+    }
+
+    // clear cache to make sure we do not mix data
     Q_FOREACH( ResourceData* data, ResourceData::allResourceData()) {
         data->invalidateCache();
     }
