@@ -19,6 +19,7 @@
 
 #include "halstorageaccess.h"
 
+#include <QtCore/QLocale>
 #include <QtCore/QDebug>
 #include <QtDBus/QDBusConnection>
 #include <QtDBus/QDBusInterface>
@@ -27,6 +28,7 @@
 #include <QtGui/QWidget>
 
 #include <unistd.h>
+#include <stdlib.h>
 
 #include "halfstabhandling.h"
 
@@ -233,6 +235,90 @@ bool StorageAccess::callHalVolumeMount()
     if (halOptions.contains("uid=")) {
         options << "uid="+QString::number(::getuid());
     }
+    //respect Microsoft Windows-enforced charsets for fat
+    if ( m_device->property("volume.fstype").toString()=="vfat" ) {
+        bool linuxMount=halOptions.contains("codepage=");
+        bool bsdMount=halOptions.contains("-D=");
+        QString codepage;
+        QString iocharset;
+        if (linuxMount)
+        {
+            codepage="codepage=";
+            iocharset="iocharset=utf8";
+        }
+        else if (bsdMount)
+        {
+            codepage="-D=CP";
+            iocharset="-L="+QLocale::system().name()+".UTF-8";
+        }
+
+        if (linuxMount||bsdMount)
+        {
+            switch (QLocale::system().language()) {
+                case QLocale::Russian:
+                case QLocale::Ukrainian:
+                case QLocale::Byelorussian:
+                case QLocale::Bulgarian:
+                    codepage+="866";
+                    break;
+                case QLocale::German:
+                case QLocale::Italian:
+                case QLocale::Spanish:
+                case QLocale::French:
+                case QLocale::Dutch:
+                case QLocale::Danish:
+                case QLocale::Swedish:
+                case QLocale::Norwegian:
+                case QLocale::Icelandic:
+                case QLocale::English:
+                    codepage+="437";
+                    break;
+                case QLocale::Portuguese:
+                    codepage+="860";
+                    break;
+                case QLocale::Hebrew:
+                    codepage+="862";
+                    break;
+                case QLocale::Turkish:
+                    codepage+="857";
+                    break;
+                case QLocale::Chinese:
+                    if (QLocale::system().country()==QLocale::China)
+                        codepage+="936";
+                    else
+                        //case QLocale::Taiwan:
+                        //case QLocale::HongKong:
+                        //case QLocale::Macau:
+                        codepage+="950";
+                    break;
+                case QLocale::Japanese:
+                    codepage+="932";
+                    break;
+                case QLocale::Korean:
+                    codepage+="949";
+                    break;
+                case QLocale::Thai:
+                    codepage+="874";
+                    break;
+                default:
+                    codepage.clear();
+            }
+            if (!codepage.isEmpty()) {
+                options << codepage;
+                options << iocharset;
+            }
+        }
+    }
+
+    // pass our locale to the ntfs-3g driver so it can translate local characters
+    if ( m_device->property("volume.fstype").toString()=="ntfs-3g" && halOptions.contains("locale=") ) {
+        // have to obtain LC_CTYPE as returned by the `locale` command
+        // check in the same order as `locale` does
+        char *cType;
+        if ( (cType = getenv("LC_ALL")) || (cType = getenv("LC_CTYPE")) || (cType = getenv("LANG")) ) {
+            options << "locale="+QString(cType);
+        }
+    }
 
     msg << "" << "" << options;
 
